//
//  WMManager.h
//  Whitemobi
//
//  Created by iMac on 08.04.16.
//  Copyright © 2016 Whitemobi. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

/*!
 * @brief A block that is used for callbacks from server.
 * @param result Indicates the request status (done or not).
 * @param response Is used for pass response data to UI.
 * @param error Is used for error handling.
 */
typedef void (^RequestCompletionHandler)(BOOL result, NSDictionary *response , NSError *error);

/*!
 * Depending on selected mode in the admin panel by the user,
 * the returned data may be different.
 * There are three modes that can be passed to the SDK
 */
typedef NS_ENUM(NSUInteger,WMOWSKDMode)
{
    /// Indicates the test mode
    WMOWSKDTestMode = 1,
    /// Indicates the safe mode
    WMOWSKDSafeMode = 2,
    /// Indicates the live mode
    WMOWSKDLiveMode = 3
};


/*!
 * There are two callback parameters that can be received from server.
 * Depending on the callback mode, rewards can be actually received by user.
 * In the client-to-server mode you should set up user_id if you want
 * to received successful callback.
 */
typedef NS_ENUM(NSUInteger,WMOWSKDCallbackMode)
{
    /// Indicates the client-to-server mode
    WMOWSKDCallbackClientToServer = 1,
    /// Indicates the server-to-server mode
    WMOWSKDCallbackServerToServer = 2
};


@protocol WMManagerDelegate;

/*!
 * @name Main class used by WhitemobiSDK
 */
@interface WMManager : NSObject

/*!
 *  @brief Optional; Sets the SDK Delegate that will receive Rewards callbacks (success, error, amount)
 *  @param delegate - (optional) reference to the object that implements WMManagerDelegate protocol
 */
@property (weak, nonatomic) id <WMManagerDelegate> delegate;

/*!
 * @brief You can get your app id on Whitemobi's dashboard: <ref to website>
 * @param applicationKey Initializes the SDK.
 */

@property (nonatomic, strong) NSString *applicationKey;

/*!
 * @discussion Optional; You should set up this parameter if you'll use clien-to-server mode.
 *             If you miss this point, the user won't get successful callback from server about
 *             completed offerwall task.
 *             You don't need it if you use server-to-server mode.
 * @param userID User unique identifier that was setuped in application.
 */
@property (nonatomic, strong) NSString *userID;

/*!
 * @brief The 'WMManager' singleton instance.
 * @return The shared 'WMManager' instance.
 */

+(instancetype) sharedManager;

/*!
 * @discussion Use this init method if you selected server-to-server mode.
 * @param applicationKey Your private application key.
 */
-(void)initWithKey:(NSString *)applicationKey;

/*!
 * @discussion Use this init method if you selected client-to-server mode.
 * @param applicationKey Your private application key.
 * @param userID Your inner userID in application.
 */
-(void)initWithKey:(NSString *)applicationKey forUser:(NSString *)userID;

/*!
 * Show an offerwall
 * @param viewController Current UIViewController which is used for calling offerwall
 */
-(void)showOfferWallInViewController:(UIViewController *)viewController;

/*!
 * @brief Use it if you need to send an extra request to check reward
 *        for completed offerwall task.
 * @warning No need to call it.
 */
-(void)needRequest;

/*!
 * @brief Received unique device identifier.
 * @return deviceID Unique device identifier.
 */
-(NSString *)deviceID;

/*!
 * @brief deviceType Constant value declared on server.
 * @return deviceType Device type.
 */
-(NSNumber *)deviceType;

/*!
 * @brief Received mobile platform.
 * @return platform Constant value declared on server.
 */
-(NSNumber *)mobilePlatform;

/*!
 *  @brief Returns the current version of the SDK.
 *  @return string SDK version.
 */
- (NSString*)SDKVersion;

@end

/*!
 * @name WMManagerDelegate
 *
 * @discussion If you use clien-to-server mode than you should to assign
 *             your class to WMManagerDelegate and implement it's method
 * @code [receiveCoinsForOffer:]
 *
 */
@protocol WMManagerDelegate <NSObject>

/*!
 * @brief Delegate method that you'd use if you select client-to-server mode
 *        to retreive reward for offerwall task.
 * @parameter offerResponse NSDictionary that contains response from server.
 */
-(void)receiveCoinsForOffer:(NSDictionary *)offerResponse;

@end
